import pytest
import yaml

from si_tests import logger
from si_tests import settings

LOG = logger.logger


@pytest.mark.parametrize("_", [f"CLUSTER_NAME={settings.TARGET_CLUSTER}"])
def test_delete_master_node(kaas_manager, show_step, _):
    """Delete/Create manager machines one by one

    Scenario:
            1. Delete manager machine
            2. Wait for machine deleted
            3. Create new manager machine
            4. Wait for machine spawned & became ready
            5. Check upgrade index
    """

    cluster_name = settings.TARGET_CLUSTER
    namespace_name = settings.TARGET_NAMESPACE

    ns = kaas_manager.get_namespace(namespace_name)
    LOG.info("Namespace name - %s", namespace_name)

    cluster = ns.get_cluster(cluster_name)
    LOG.info("Cluster name - %s", cluster_name)

    region = cluster.region_name
    machines = cluster.get_machines(machine_type='control')

    for machine in machines:
        LOG.info("Process machine - %s", machine.name)

        machine_name = machine.name
        node_name = machine.get_k8s_node_name()
        machine_data = cluster.get_existing_machine_spec(machine)

        show_step(1)
        machine.delete()

        show_step(2)
        cluster.wait_machine_deletion(machine_name, retries=60)
        cluster.check.check_deleted_node(node_name)

        show_step(3)
        new_machine = cluster.create_machine(
            node_type="master",
            region=region,
            **machine_data)

        show_step(4)
        cluster.check.check_created_machine(new_machine)

        show_step(5)
        # Check new index is greater than max existed
        LOG.info(f"Check new machine {new_machine.name} upgrade index value")
        current_index = cluster.get_machine(new_machine.name).get_upgrade_index()
        LOG.info(f"New machine {new_machine.name} upgrade index is: {current_index}")
        all_indexes = {machine.name: machine.get_upgrade_index() for machine in cluster.get_machines() if
                       machine.name != new_machine.name}
        fail_message = (f"Wrong index {current_index} for machine {new_machine.name}. "
                        f"New index should be greater than {max(all_indexes.values())}. "
                        f"Existed indexes excluding new machine:\n {yaml.dump(all_indexes)}")
        assert current_index > max(all_indexes.values()), fail_message
